using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using Ares.CommonObjects;
using Ares.PacketHandlers;
using Ares.Crypt;
using ZLib;

namespace Ares.Protocols
{
    class Hashlinks
    {
        public static String EncodeHashlink(ChannelListItem item)
        {
            AresDataPacket packet = new AresDataPacket();
            packet.WriteBytes(new byte[20]);
            packet.WriteString("CHATCHANNEL");
            packet.WriteIP(item.externalIp);
            packet.WriteInt16(item.port);
            packet.WriteIP(item.localIp);
            packet.WriteString(item.name);
            packet.WriteByte(0);
            byte[] bytes = packet.ToByteArray();
            bytes = Zlib.Compress(bytes);
            bytes = AresCrypt.e67(bytes, 28435);
            return Convert.ToBase64String(bytes);
        }

        public static ChannelListItem DecodeHashlink(String item)
        {
            ChannelListItem chitem = new ChannelListItem();

            item = item.Trim();
            if (item.StartsWith("arlnk://")) item = item.Substring(8);

            try
            {
                if (item.ToUpper().StartsWith("CHATROOM:")) // not encrypted
                {
                    item = item.Substring(9);
                    int split = item.IndexOf(":");
                    chitem.externalIp = IPAddress.Parse(item.Substring(0, split));
                    chitem.localIp = IPAddress.Parse(item.Substring(0, split));
                    item = item.Substring(split + 1);
                    split = item.IndexOf("|");
                    chitem.port = ushort.Parse(item.Substring(0, split));
                    chitem.name = item.Substring(split + 1);
                    chitem.topic = String.Empty;
                    chitem.users = 0;
                    return chitem;
                }
                else // encrypted
                {
                    byte[] bytes = Convert.FromBase64String(item);
                    bytes = AresCrypt.d67(bytes, 28435);
                    bytes = Zlib.Decompress(bytes, false);
                    AresDataPacket packet = new AresDataPacket(bytes);
                    packet.SkipBytes(32);
                    chitem.externalIp = packet.ReadIP();
                    chitem.port = packet.ReadInt16();
                    chitem.localIp = packet.ReadIP();
                    chitem.name = packet.ReadString();
                    chitem.topic = String.Empty;
                    chitem.users = 0;
                    return chitem;
                }
            }
            catch // badly formed hashlink
            {
                return null;
            }
        }
    }
}
