using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Xml;
using System.Threading;

namespace cb0tServer
{
    class RangeBans
    {
        private static Mutex mutex = new Mutex();
        private static List<String> banned_items = new List<String>();

        public static void AddBan(String ip)
        {
            ip = ip.Replace("*", "");

            foreach (String a in banned_items)
            {
                if (ip == a)
                {
                    return;
                }
            }

            banned_items.Add(ip);
            SaveBans();
        }

        public static String RemoveBan(ushort _id)
        {
            if (_id < banned_items.Count)
            {
                String str = banned_items[_id];
                banned_items.RemoveAt(_id);
                SaveBans();
                return str;
            }

            return null;
        }

        public static String[] GetBans()
        {
            if (banned_items.Count > 0)
            {
                return banned_items.ToArray();
            }

            return null;
        }

        public static bool IsBanned(UserObject userobj)
        {
            foreach (String a in banned_items)
            {
                if (userobj.physical_ip.ToString().StartsWith(a))
                {
                    return true;
                }
            }

            return false;
        }

        public static void ClearBans()
        {
            banned_items.Clear();
            SaveBans();
        }

        public static void LoadBans()
        {
            banned_items.Clear();

            if (!File.Exists(sb0t_winGUI.RunOnceFileUpdater.sbotpath + "data/rangebans.xml")) return;

            try
            {
                FileStream f = new FileStream(sb0t_winGUI.RunOnceFileUpdater.sbotpath + "data/rangebans.xml", FileMode.Open);
                XmlReader xml = XmlReader.Create(new StreamReader(f));

                xml.MoveToContent();
                xml.ReadSubtree().ReadToFollowing("rangebans");

                while (xml.ReadToFollowing("rangeban"))
                {
                    xml.ReadSubtree().ReadToFollowing("ip");
                    banned_items.Add(Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString())));
                }

                xml.Close();
                f.Flush();
                f.Close();
            }
            catch { }
        }

        private static void SaveBans()
        {
            mutex.WaitOne();

            try
            {
                XmlWriterSettings appearance = new XmlWriterSettings();
                appearance.Indent = true;
                XmlWriter xml = XmlWriter.Create(sb0t_winGUI.RunOnceFileUpdater.sbotpath + "data/rangebans.xml", appearance);

                xml.WriteStartDocument();
                xml.WriteStartElement("rangebans");

                foreach (String a in banned_items)
                {
                    xml.WriteStartElement("rangeban");
                    xml.WriteElementString("ip", Convert.ToBase64String(Encoding.UTF8.GetBytes(a)));
                    xml.WriteEndElement();
                }

                xml.WriteEndElement();
                xml.WriteEndDocument();
                xml.Flush();
                xml.Close();
            }
            catch { }

            mutex.ReleaseMutex();
        }
    }
}
